/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _CAMERA_H_
#define _CAMERA_H_

#include "glUtil.h" 
#include "vector3.h"

const float STEP_ANGLE = 0.05;


class Camera
{
public:
	Camera(float dist, float fov, float nearClip, float farClip, int xWin, int yWin, int width, int height);
	Camera(Vector3 position, Vector3 target, Vector3 up, float fov, float nearClip, float farClip, int xWin, int yWin, int width, int height);

	void setPositionAndAxis(Vector3 position, Vector3 target, Vector3 up);
		
	void setFarClip(float farClip);
		
	float getNearClip();
	float getFarClip();

	Vector3 getPosition();
	Vector3 getDirection();
	float getVAngle();
	float getHAngle();
		
	Vector3 getTranslatedPosition(Vector3 tr, Vector3 vertical);
		
	Vector3 getXAxis();
	Vector3 getYAxis();
	Vector3 getZAxis();	
		
	void getDisplayParam(int &xWin, int &yWin, int &w, int &h);
		
	int getLeft();
	int getRight();
	int getBottom();
	int getTop();
		
	void perspective();
	void place();
	void lookAtTheSky();
	void initModelview();
	void initProjection();
	void initPickingProjection(int xMouse, int yMouse, float delX, float delY);
	
	void setPosition(Vector3 position);
	void translate(Vector3 tr);
	void translate(Vector3 tr, Vector3 vertical);
	void rotate(float phi, float theta);
	void rotate(float phi, float theta, Vector3 vertical);
	void keplerRotate(float hAngle, float vAngle); //for Kepler-Bubble
	void rotAroundO(bool absoluteYAxis, float dist, float hAngle, float vAngle, float limitUp, float limitDown);
	void setPositionLookingAtO(Vector3 position);
	void setzAxisLookingAtO(Vector3 z);
	void setDistToO(float dist);
		
private:
	Vector3 mPosition;
	Vector3 xAxis;
	Vector3 yAxis;
	Vector3 zAxis;
	
	float mFov;
	float mNearClip;
	float mFarClip;
			
	int xWindow;
	int yWindow;
	int mWidth;
	int mHeight;
};

#endif // _CAMERA_H_
