/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "gridCubeTTT.h"


GridCubeTTT::GridCubeTTT(int size, std::vector<int> colors, float spheresRadius, float cylinderRadius, float interval):
	GridCube (size, colors, interval, cylinderRadius) {

	SphereMesh *sphereMesh = new SphereMesh(spheresRadius, 12, 8, false);
		
	mainEntitiesIndex = 0;
	EntityCubeTab *tabSpheres = new EntityCubeTab(size, interval, true, materials[0], sphereMesh, 
							 					Vector3(1., 0., 0.), Vector3(0.,1.,0.), Vector3(0.,0.,1.),
							 					Vector3(-interval * ((float) size - 1.) / 2., -interval * ((float) size - 1.) / 2., -interval * ((float) size - 1.) / 2.),
												Vector3(0.,1.,0.), 0.);
		
	tabSpheres->setClickable();
	addChild(tabSpheres, mainEntitiesIndex);
				
	CylinderMesh *rodsAxisMesh = new CylinderMesh (((float) size - 1.) * interval, 1, cylinderRadius, 5, false); 
		
	rodsXindex = 1;
	addChild(new EntityPlane (size, interval, true, materials[0], rodsAxisMesh,
							Vector3(0.,1.,0.), Vector3(0.,0.,1.),
							Vector3(0., -interval * ((float) size - 1.) / 2., -interval * ((float) size - 1.) / 2.),
							Vector3(0.,1.,0.), 0.),
			 rodsXindex);								   
		
	rodsYindex = 2;
	addChild(new EntityPlane (size, interval, true, materials[0], rodsAxisMesh,
							Vector3(0.,1.,0.), Vector3(0.,0.,1.),
							Vector3(-interval * ((float) size - 1.) / 2., 0., -interval * ((float) size - 1.) / 2.),
							Vector3(0.,0.,1.), -90.),
			 rodsYindex);	

	rodsZindex = 3;
	addChild(new EntityPlane (size, interval, true, materials[0], rodsAxisMesh,
							Vector3(0.,1.,0.), Vector3(0.,0.,1.),
							Vector3(-interval * ((float) size - 1.) / 2., -interval * ((float) size - 1.) / 2., 0.),
							Vector3(0.,1.,0.), 90.),
			 rodsZindex);	
	
}

GridCubeTTT::~GridCubeTTT() {
	
}



void GridCubeTTT::reinit() {
	EntityCubeTab *tabSpheres = dynamic_cast <EntityCubeTab*> (children[mainEntitiesIndex]);
	tabSpheres->setValueAndMaterialForAllEntities(0, materials[0]);
	
	EntityPlane *axis;
	axis = dynamic_cast <EntityPlane*> (children[rodsXindex]);	
	axis->setValueAndMaterialForAllEntities(0, materials[0]);
	axis = dynamic_cast <EntityPlane*> (children[rodsYindex]);	
	axis->setValueAndMaterialForAllEntities(0, materials[0]);
	axis = dynamic_cast <EntityPlane*> (children[rodsZindex]);	
	axis->setValueAndMaterialForAllEntities(0, materials[0]);	
	
	for(std::map<int, Entity*>::iterator it = children.begin(); it!=children.end(); it++) {
		int key = (*it).first;
		if(key<0) removeChild(key);
	}		
}


int GridCubeTTT::updateAxis(int player, int i, int j, int k) {
	int res = 0;
	EntityCubeTab *tab = dynamic_cast <EntityCubeTab*> (children[mainEntitiesIndex]);	
	EntityPlane *axis;	


	if(tab->testAxis(1, j, k, player) == mSize){
		axis = dynamic_cast <EntityPlane*> (children[rodsXindex]);	
		axis->setValueAndMaterial(player, materials[2*player-1], j*mSize+k);
		res ++;
	}

	if(tab->testAxis(2, k, i, player) == mSize){
		axis = dynamic_cast <EntityPlane*> (children[rodsYindex]);	
		axis->setValueAndMaterial(player, materials[2*player-1], i*mSize+k);
		res ++;
	}
	
	if(tab->testAxis(3, i, j, player) == mSize){
		axis = dynamic_cast <EntityPlane*> (children[rodsZindex]);	
		axis->setValueAndMaterial(player, materials[2*player-1], j*mSize+i);
		res ++;
	}	
	return res;
}

