/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _PRISM_H_
#define _PRISM_H_

#include <math.h> 
#include "../newtEngine/entity.h"
#include "../newtEngine/mesh.h"

#include "polygon.h"

 
enum baseType {BASE_NONE, BASE_FLAT, BASE_FLAT2, BASE_DOME};


class Prism: public virtual Entity
{
public:	
	Prism(int nbSides, float height, int nbH, float innerRadius, int nbS, float edgeRadius, int nbR,
		  float baseTexOffset, baseType base1, baseType base2, bool inside, 
		  Material *materialSides, Material *materialBase1, Material *materialBase2);
				
	~Prism();
	
	void setMaterialSides(Material *material);
	void setMaterialBase(Material *material, int base);
		
		
protected:
	float mHeight, mInnerRadius, mEdgeRadius;
	int mNbSides;
	int mNbH, mNbS, mNbR;
};


class PrismWithPosition: public EntityWithPosition, public Prism
{
public:	
	PrismWithPosition(int nbSides, float height, int nbH, float innerRadius, int nbS, float edgeRadius, int nbR,
					  float baseTexOffset, baseType base1, baseType base2, bool inside,
					  Material *materialSides, Material *materialBase1, Material *materialBase2);
	~PrismWithPosition();
				
	//void applyTransformation();
	
};


class PrismMesh: public virtual Mesh
{
public:
	PrismMesh(int nbSides, float height, int nbH, float innerRadius, int nbS, float edgeRadius, int nbR,
		  	  float baseTexOffset, baseType base1, baseType base2, bool inside);

	~PrismMesh();		

	void initFirstParams();				
		
	void createVertices(Vector3 *ptr);
	void createColors(Color *ptr);
	void createNormals(Vector3 *ptr);
	void createTexCoords(GLfloat *ptr);
	void createFogCoords(GLfloat *ptr);
	void createTangents(Vector3 *ptr);			
	
	void createIndices(int numArray, GLuint *ptr);		

		
		
protected:
	float mHeight, mInnerRadius, mEdgeRadius;
	int mNbSides;
	int mNbH, mNbS, mNbR;
	bool mInside;		
	baseType mBase1, mBase2;
};

#endif // _PRISM_H_
